import copy

from typing import Union, List, Tuple, Any, Optional

COLLECTION: Any = Union[List, Tuple]


def index_of(
        array: COLLECTION,
        item: Any,
        start: int = 0,
        end: Optional[int] = None
) -> int:
    if end is None: end = len(array)
    try:
        return array.index(item, start, end)
    except:
        return -1


def replace_all(array: List, _old: Any, _new: Any) -> List:
    _array: List = copy.deepcopy(array)
    for i in range(len(_array)):
        if _array[i] == _old:
            _array[i] = _new
    return _array



from typing import Optional, List, Dict, Union, Any

from project_utils.exception import DjangoWebException

DATATYPE = Optional[Union[List, Dict]]
ERRTYPE = Optional[DjangoWebException]
BODYTYPE = Dict[str, Any]



import os


def mkdir(path: str) -> bool:
    try:
        os.mkdir(path)
    except:
        return False
    else:
        return True


def remove(path: str):
    def remove_inner(p: str):
        try:
            os.remove(p)
        except:
            pass

    # 如果被删除的路径是文件夹
    if os.path.isdir(path):
        # 文件夹不为空
        if len(os.listdir(path)) > 0:
            for file in os.listdir(path):
                # 删除文件夹内部的文件
                remove_inner(os.path.join(path, file))
    # 删除操作
    remove_inner(path)


def is_empty(path: str) -> int:
    if len(os.listdir(path)) > 0:
        return len(os.listdir(path))
    else:
        return 0



from ._async_view import AsyncAPIView
from ._swagger_base import SwaggerTypeBase

from ._base_api import base_api
from ._rest_view import BaseView, AsyncBaseView
from ._user_view import BaseUserLoginView, BaseUserModel, AsyncBaseUserLoginView

AsyncView = AsyncAPIView
SwaggerType = SwaggerTypeBase

__all__ = [
    "base_api",
    "BaseView",
    "AsyncView",
    "AsyncBaseView",
    "BaseUserLoginView",
    "BaseUserModel",
    "AsyncBaseUserLoginView",
    "SwaggerType"
]



from abc import ABCMeta
from typing import Union, Tuple, Dict

from django.db.models import Model


class BaseMeta(metaclass=ABCMeta):
    model: Model
    fields: Union[str, Tuple] = "__all__"
    exclude: Tuple = ()
    read_only_fields: Tuple = ("id",)
    extra_kwargs: Dict = {}



from abc import ABCMeta

from typing import List, Dict, Union, Optional

from drf_yasg.openapi import Parameter, Schema

from project_utils.web.django.swagger import ResponseType, ResponseSchema, ParamsTypes


class SwaggerTypeBase(metaclass=ABCMeta):
    request_get: List[Parameter]
    request_post: Schema
    request_put: Schema
    request_delete: Dict[str, Union[List[Parameter], Schema, None]] = {
        "query": [],
        "params": None
    }

    def __init__(
            self, *,
            request_get: Optional[List[Parameter]] = None,
            request_post: Optional[Schema] = None,
            request_put: Optional[Schema] = None,
            request_delete: Optional[Dict[str, Union[List[Parameter], Schema, None]]] = None,
    ):
        self.request_get = request_get
        self.request_post = request_post
        self.request_put = request_put
        if request_delete:
            self.request_delete = request_delete

    def response(self, properties: Optional[Dict[str, Schema]] = None) -> Dict[Union[str, int], Schema]:
        return {
            200: ResponseType().success(properties=properties),
            400: ResponseType().error()
        }



from .. import DjangoWebException


class ViewException(DjangoWebException):
    error_id = "ca8ceb45cc3e48acbb872f7a12864f92"
    status: int

    def __init__(self, status: int = 400, *args, **kwargs):
        self.status = status
        super().__init__(*args, **kwargs)



from ._pool import MysqlPool
from ._result import MysqlResult

mysql_pool = pool = MysqlPool
mysql_result = result = MysqlResult

__all__ = [
    "mysql_pool",
    "pool",
    "MysqlPool",
    "mysql_result",
    "result",
    "MysqlResult"
]



from ._base_model import BaseModel
from ._base_meta import BaseMeta
from ._id_model import IDModel
from ._model_type import Int, Bool, Byte, Float, String, Long, File, Email, Url, DateTime, Json

Model = BaseModel

__all__ = [
    "Model",
    "BaseModel",
    "BaseMeta",
    "IDModel",
    "Int",
    "Bool",
    "Byte",
    "Float",
    "String",
    "Long",
    "File",
    "Email",
    "DateTime",
    "Json"
]



from . import mysql_config, redis_config, kafka_config, hbase_config

mysql = Mysql = MysqlConfig = mysql_config = mysql_config.MysqlConfig
redis = Redis = RedisConfig = redis_config = redis_config.RedisConfig
kafka = Kafka = KafkaConfig = kafka_config = kafka_config.KafkaConfig
hbase = Hbase = HbaseConfig = hbase_config = hbase_config.HbaseConfig

__all__ = [
    "mysql",
    "Mysql",
    "MysqlConfig",
    "mysql_config",
    "redis",
    "Redis",
    "RedisConfig",
    "redis_config",
    "kafka",
    "Kafka",
    "KafkaConfig",
    "kafka_config",
    "hbase",
    "Hbase",
    "HbaseConfig",
    "hbase_config"
]



from typing import List, Optional, Dict, Union

from .base_config import BaseConfig


class KafkaConfig(BaseConfig):
    bootstrap_servers: List[str]
    topic: str
    group: Optional[str]

    def __init__(self, bootstrap_servers: str, topic: str, group: Optional[str] = None):
        super().__init__(port="0")
        self.bootstrap_servers = bootstrap_servers.split(";")
        self.topic = topic
        if group: self.group = group

    def to_dict(self) -> Dict[str, Union[str, int, List[str]]]:
        result: Dict[str, Union[str, int, List[str]]] = {
            "bootstrap_servers": self.bootstrap_servers,
            "topic": self.topic,
        }
        if self.group:
            result['group'] = self.group
        return result



from typing import Optional, Union, Any, Dict

from drf_yasg.openapi import Schema, Parameter, IN_QUERY

from ._base import SwaggerParamsBase
from ._types import SwaggerParamsEnum


class SwaggerQueryParams(SwaggerParamsBase):
    name: str
    type: Union[SwaggerParamsEnum, Any]
    require: bool
    description: Optional[str]

    def __init__(self, name: str, require: bool = False,
                 description: Optional[str] = None):
        self.name = name
        self.type = SwaggerParamsEnum.string
        self.require = require
        self.description = description

    def params(self, properties: Optional[Dict[str, Schema]] = None, items: Schema = None) -> Union[Schema, Parameter]:
        return Parameter(
            name=self.name,
            in_=IN_QUERY,
            description=self.description,
            required=self.require,
            type=self.type
        )



from ._params import SwaggerParams
from ._query import SwaggerQueryParams
from ._response import SwaggerParamsResponse
from ._types import SwaggerParamsEnum

params = Params = SwaggerParams
query = Query = SwaggerQueryParams
response_type = ResponseType = SwaggerParamsResponse
response_schema = ResponseSchema = SwaggerParamsResponse.inner
params_types = ParamsTypes = SwaggerParamsEnum

__all__ = [
    "params",
    "Params",
    "query",
    "Query",
    "response_type",
    "ResponseType",
    "response_schema",
    "ResponseSchema",
    "params_types",
    "ParamsTypes",
]



from ._base_view import BaseRestAPIView
from ._async_base_view import AsyncBaseAPIView

BaseView = RestAPIView = RestView = BaseRestAPIView
AsyncBaseView = AsyncBaseAPIView

__all__ = [
    "BaseView",
    "RestAPIView",
    "RestView",
    "BaseRestAPIView",
    "AsyncBaseView",
    "AsyncBaseAPIView",
]



import asyncio

from abc import ABCMeta, abstractmethod

from typing import Optional, Callable

from django.http.request import HttpRequest
from django.http.response import HttpResponse
from django.utils.deprecation import MiddlewareMixin

from project_utils.web.django.conf import DjangoConfig


class BaseMiddleware(MiddlewareMixin, metaclass=ABCMeta):
    config: DjangoConfig

    @abstractmethod
    def process_request(self, request: HttpRequest) -> Optional[HttpResponse]:
        return

    def process_view(self, request: HttpRequest, callback: Callable, callback_args: ..., callback_kwargs: ...) -> \
            Optional[HttpResponse]:
        return callback(request, *callback_args, **callback_kwargs)

    @abstractmethod
    def process_response(self, request: HttpRequest, response: HttpResponse) -> HttpResponse:
        return response

    @abstractmethod
    def process_exception(self, request: HttpRequest, exception: Exception) -> HttpResponse:
        return HttpResponse(str(exception))

    @abstractmethod
    def process_template_response(self, request: HttpRequest, response: HttpResponse) -> HttpResponse:
        return response



from enum import Enum

from drf_yasg import openapi


class SwaggerParamsEnum(Enum):
    object: str = openapi.TYPE_OBJECT
    string: str = openapi.TYPE_STRING
    number: str = openapi.TYPE_NUMBER
    integer: str = openapi.TYPE_INTEGER
    boolean: str = openapi.TYPE_BOOLEAN
    array: str = openapi.TYPE_ARRAY
    file: str = openapi.TYPE_FILE


if __name__ == '__main__':
    print(SwaggerParamsEnum.file)



from setuptools import setup, find_packages

VERSION = "0.3.8"
DESCRIPTION = "python project_utils tools"
setup(
    name="project-utils-2023",
    version=VERSION,
    author="mylx2014",
    author_email="mylx2014@163.com",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=open('README.md', encoding="UTF8").read(),
    packages=find_packages(),
    install_requires=['loguru', 'dbutils', 'pymysql'],
    keywords=['python', 'utils', 'project utils', "aiofiles"],
    data_files=[],
    entry_points={},
    license="MIT",
    url="https://gitee.com/mylx2014/project-utils.git",
    scripts=[],
    classifiers=[
        "Programming Language :: Python :: 3.8",
    ]
)



import jwt
import json
import traceback

from abc import ABCMeta, abstractmethod
from typing import Optional

from project_utils.exception import ViewException

from ..conf import DjangoConfig
from ..request import WebRequest
from ._types import REQUEST_TYPE, PARAMS_TYPE, USER_TYPE


class Base(metaclass=ABCMeta):
    request: REQUEST_TYPE
    method: str
    config: DjangoConfig

    def __init__(self, request: REQUEST_TYPE, method: str, config: DjangoConfig):
        self.request = request
        self.method = method
        self.config = config

    @abstractmethod
    def parse_params(self) -> PARAMS_TYPE:
        pass

    def parse_token(self, request: WebRequest) -> USER_TYPE:
        for path in self.config.system_path:
            if path and path in request.path:
                return {}
        session_token: Optional[str] = request.session.get("usertoken")
        request_token: Optional[str] = request.meta.get("HTTP_AUTHORIZATION")
        assert session_token and request_token, ViewException("Input token value isn't null!", "token为空")
        assert session_token == request_token, ViewException("Input token value and session token isn't match!")
        token = session_token = request_token
        print(token)
        user_body: str = jwt.decode(token, key=self.config.settings.SECRET_KEY, algorithms="HS256")['user']
        return json.loads(user_body)

    def request_init(self, config: DjangoConfig) -> WebRequest:
        request: WebRequest = WebRequest(request=self.request, config=config)
        try:
            params: PARAMS_TYPE = self.parse_params()
        except Exception as e:
            raise ViewException(error_summary=str(e), error_detail=traceback.format_exc())
        request.params = params
        try:
            user: USER_TYPE = self.parse_token(request)
        except Exception as e:
            raise ViewException(error_summary=str(e), error_detail=traceback.format_exc())
        request.user = user
        return request

    async def async_request_init(self, config: DjangoConfig):
        return self.request_init(config)



import jwt
import json
import uuid
import base64
import traceback

from abc import ABCMeta

from typing import List, Optional, Dict, Any, Union

from django.conf import settings
from django.contrib.auth import authenticate

from rest_framework.views import APIView
from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework.serializers import ReturnDict

from project_utils.time import compute_day
from project_utils.exception import DjangoWebException, ViewException
from project_utils.web.django.conf import DjangoConfig
from project_utils.web.django.serializer import BaseSerializer
from project_utils.web.django.response import WebResponse

from .._async_view import AsyncAPIView

from ._model import BaseUserModel


class BaseUserLoginView(APIView, metaclass=ABCMeta):
    config: DjangoConfig
    model: BaseUserModel
    serializer: BaseSerializer

    require: List[str] = ["username", "password", "checkcode"]
    pop_field: List[str] = ["password", "last_login", "is_superuser", "first_name", "last_name", "is_staff",
                            "is_active", "date_joined", "is_delete", "groups", "user_permissions"]

    def check(self, params: Dict[str, Any], field_list: List[str]):
        for field in field_list:
            assert field in params and params.get(field) is not None, Exception(
                f"Field name's {field} is not in request field list or name's {field} value is null!")

    def show_exception(self, exception: DjangoWebException):
        self.config.printf.error(exception.error_detail)
        self.config.printf.warning(exception.error_summary)

    def create_exception(self, status, e: Exception):
        return ViewException(status, str(e), traceback.format_exc())

    def before_login(self, request: Request, params: Dict[str, Any]):
        username: str = params.get("username")
        password: str = params.get("password")
        checkcode: str = params.get("checkcode")
        usercode: Optional[str] = request.session.get("user_code")
        assert usercode, Exception("Session save user code value's null!")
        localcode: str = self.UserCode(username, password, usercode).encode()
        assert checkcode == localcode, Exception("Local save code and user input code not match!")
        return self.UserInfo(username, password)

    def logined(self, request: Request, serializer: BaseSerializer) -> str:
        data: ReturnDict = serializer.data.copy()
        exp: float = compute_day(7).timestamp()
        payload: Dict[str, Union[str, float]] = {"exp": exp, "user": json.dumps(data)}
        token: str = jwt.encode(payload, settings.SECRET_KEY, algorithm="HS256")
        request.session['usertoken'] = token
        return token

    def pop(self, data: ReturnDict, pop_field) -> ReturnDict:
        for field in pop_field:
            data.pop(field)
        return data

    def get(self, request: Request, id: Optional[str] = None) -> Response:
        self.config.printf.info(f"request success!\npath is {request.path}\nmethod is get\nselect id is {id}")
        user_code: str = uuid.uuid4().hex[:12]
        request.session["user_code"] = user_code
        self.config.printf.success(f"current request user code's {user_code}")
        return WebResponse(data={"code": user_code}).to_rest_response()

    def post(self, request: Request) -> Response:
        self.config.printf.info(f"request success!\npath is {request.path}\nmethod is post")
        params: Dict[str, Any] = request.data
        try:
            self.check(params, self.require)
        except Exception as e:
            _e: DjangoWebException = self.create_exception(403, e)
            self.show_exception(_e)
            return WebResponse(status=403, error=_e).to_rest_response()
        try:
            user_info: BaseUserLoginView.UserInfo = self.before_login(request, params)
        except Exception as e:
            _e: DjangoWebException = self.create_exception(403, e)
            self.show_exception(_e)
            return WebResponse(status=403, error=_e).to_rest_response()
        user: Optional[BaseUserModel] = authenticate(**user_info.to_dict())
        try:
            assert user, Exception("Input username and password not match!")
        except Exception as e:
            _e: DjangoWebException = self.create_exception(403, e)
            self.show_exception(_e)
            return WebResponse(status=403, error=_e).to_rest_response()
        serializer: BaseSerializer = self.serializer(instance=user)
        try:
            token: str = self.logined(request, serializer)
        except Exception as e:
            _e: DjangoWebException = self.create_exception(410, e)
            self.show_exception(_e)
            return WebResponse(status=410, error=_e).to_rest_response()
        result: ReturnDict = self.pop(serializer.data.copy(), self.pop_field)
        return WebResponse(data={"user_info": result, "user_token": token}).to_rest_response()

    class UserCode:
        username: str
        password: str
        checkcode: str

        def __init__(self, username: str, password: str, checkcode: str):
            self.username = username
            self.password = password
            self.checkcode = checkcode

        def encode(self) -> str:
            code: str = "_".join([self.username, self.checkcode, self.password])
            return base64.b64encode(code.encode("utf-8")).decode("utf-8")

    class UserInfo:
        username: str
        password: str

        def __init__(self, username: str, password: str):
            self.username = username
            self.password = password

        def to_dict(self):
            return {
                "username": self.username,
                "password": self.password
            }


class AsyncBaseUserLoginView(AsyncAPIView, metaclass=ABCMeta):
    config: DjangoConfig
    model: BaseUserModel
    serializer: BaseSerializer

    require: List[str] = ["username", "password", "checkcode"]
    pop_field: List[str] = ["password", "last_login", "is_superuser", "first_name", "last_name", "is_staff",
                            "is_active", "date_joined", "is_delete", "groups", "user_permissions"]

    async def check(self, params: Dict[str, Any], field_list: List[str]):
        for field in field_list:
            assert field in params and params.get(field) is not None, Exception(
                f"Field name's {field} is not in request field list or name's {field} value is null!")

    async def show_exception(self, exception: DjangoWebException):
        self.config.printf.error(exception.error_detail)
        self.config.printf.warning(exception.error_summary)

    async def create_exception(self, status: int, e: Exception):
        return ViewException(status, str(e), traceback.format_exc())

    async def before_login(self, request: Request, params: Dict[str, Any]):
        username: str = params.get("username")
        password: str = params.get("password")
        checkcode: str = params.get("checkcode")
        usercode: Optional[str] = request.session["user_code"]
        assert usercode, Exception("Session save user code value's null!")
        localcode: str = await self.UserCode(username, password, usercode).encode()
        assert checkcode == localcode, Exception("Local save code and user input code not match!")
        return self.UserInfo(username, password)

    async def logined(self, request: Request, serializer: BaseSerializer) -> str:
        data: ReturnDict = serializer.data.copy()
        exp: float = compute_day(7).timestamp()
        payload: Dict[str, Union[str, float]] = {"exp": exp, "user": json.dumps(data)}
        token: str = jwt.encode(payload, settings.SECRET_KEY, algorithm="HS256")
        request.session['usertoken'] = token
        return token

    async def pop(self, data: ReturnDict, pop_field) -> ReturnDict:
        for field in pop_field:
            data.pop(field)
        return data

    async def get(self, request: Request, id: Optional[str] = None) -> Response:
        self.config.printf.info(f"request success!\npath is {request.path}\nmethod is get\nselect id is {id}")
        user_code: str = uuid.uuid4().hex[:12]
        request.session["user_code"] = user_code
        self.config.printf.success(f"current request user code's {user_code}")
        return WebResponse(data={"code": user_code}).to_rest_response()

    async def post(self, request: Request) -> Response:
        self.config.printf.info(f"request success!\npath is {request.path}\nmethod is post")
        params: Dict[str, Any] = request.data
        try:
            await self.check(params, self.require)
        except Exception as e:
            _e: DjangoWebException = await self.create_exception(403, e)
            await self.show_exception(_e)
            return WebResponse(status=403, error=_e).to_rest_response()
        try:
            user_info: AsyncBaseUserLoginView.UserInfo = await self.before_login(request, params)
        except Exception as e:
            _e: DjangoWebException = await self.create_exception(403, e)
            await self.show_exception(_e)
            return WebResponse(status=403, error=_e).to_rest_response()
        user: Optional[BaseUserModel] = authenticate(**await user_info.to_dict())
        try:
            assert user, Exception("Input username and password not match!")
        except Exception as e:
            _e: DjangoWebException = await self.create_exception(403, e)
            await self.show_exception(_e)
            return WebResponse(status=403, error=_e).to_rest_response()
        serializer: BaseSerializer = self.serializer(instance=user)
        try:
            token: str = await self.logined(request, serializer)
        except Exception as e:
            _e: DjangoWebException = await self.create_exception(410, e)
            await self.show_exception(_e)
            return WebResponse(status=410, error=_e).to_rest_response()
        result: ReturnDict = await self.pop(serializer.data.copy(), self.pop_field)
        return WebResponse(data={"user_info": result, "user_token": token}).to_rest_response()

    class UserCode:
        username: str
        password: str
        checkcode: str

        def __init__(self, username: str, password: str, checkcode: str):
            self.username = username
            self.password = password
            self.checkcode = checkcode

        async def encode(self) -> str:
            code: str = "_".join([self.username, self.checkcode, self.password])
            return base64.b64encode(code.encode("utf-8")).decode("utf-8")

    class UserInfo:
        username: str
        password: str

        def __init__(self, username: str, password: str):
            self.username = username
            self.password = password

        async def to_dict(self):
            return {
                "username": self.username,
                "password": self.password
            }



import time

from datetime import datetime

from project_utils.time import datetime_to_str


class IDModel:
    id: int = 100000

    @classmethod
    def create_id(cls, table_id: int = 1000) -> str:
        left: str = str(time.time())[:6]
        center: str = datetime_to_str(datetime=datetime.now(), format="%Y%m%d%H%M%S")
        right: str = str(cls.id)
        cls.id += 1
        if cls.id >= 999999:
            cls.id = 100000
        return f"{left}{center}{table_id}{right}"
